package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.service.rank.RankService;
import com.tzld.piaoquan.recommend.server.service.recall.strategy.FestivalRecallStrategyV1;
import com.tzld.piaoquan.recommend.server.service.recall.strategy.ReturnVideoRecallStrategy;
import com.tzld.piaoquan.recommend.server.service.recall.strategy.SimHotVideoRecallStrategy;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sunxy
 */
@Service
@Slf4j
public class FestivalStrategy4RankModel extends RankService {

    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        List<Video> rovRecallRank = new ArrayList<>();
        rovRecallRank.addAll(extractAndSort(param, SimHotVideoRecallStrategy.PUSH_FORM));
        rovRecallRank.addAll(extractAndSort(param, ReturnVideoRecallStrategy.PUSH_FORM));
        rovRecallRank.addAll(extractAndSort(param, FestivalRecallStrategyV1.PUSH_FORM));
        removeDuplicate(rovRecallRank);
        // 融合排序
        List<String> videoIdKeys = rovRecallRank.stream()
                .map(t -> param.getRankKeyPrefix() + t.getVideoId())
                .collect(Collectors.toList());
        List<String> videoScores = redisTemplate.opsForValue().multiGet(videoIdKeys);
//        log.info("FestivalStrategy4RankModel mergeAndRankRovRecall videoIdKeys={}, videoScores={}", JSONUtils.toJson(videoIdKeys),
//                JSONUtils.toJson(videoScores));
        if (CollectionUtils.isNotEmpty(videoScores)
                && videoScores.size() == rovRecallRank.size()) {
            for (int i = 0; i < videoScores.size(); i++) {
                rovRecallRank.get(i).setSortScore(NumberUtils.toDouble(videoScores.get(i), 0.0));
            }
            Collections.sort(rovRecallRank,
                    Comparator.comparingDouble(o -> -(o.getSortScore() * 0.4 * (o.getRovScore() / 100))));
        }

        return rovRecallRank;
    }
}
